__author__ = 'ggegiya1'


class TestFailure(AssertionError):
    def __init__(self, message):
        super(TestFailure, self).__init__(message)


class SmartTester:
    def __init__(self):
        self.argument = None
        self.extra_arguments = None
        self.validation_function = None

    def assertThat(self, arg=None, *arguments):
        ## if the argument is a validation function
        ## then validation function result will be validated
        if hasattr(arg, "__call__"):
            self.validation_function = arg
            self.extra_arguments = arguments
        else:
            self.argument = arg
        return self

    def equalsTo(self, argument2):
        """
        Embedded method to compare the passed argument to the self.argument
        :param argument2:
        :return: True if arguments are equal, else throw an error
        """
        if self.argument == argument2:
            return True
        self.throw_error("%s and %s are not equal"
                   % (self._short(self.argument),
                      self._short( argument2)))

    def contains(self, argument2):
        if (not hasattr(self.argument, "__iter__")) and (type(self.argument) not in [str]):
            raise ValueError("The first argument is not iterable")
        if argument2 in self.argument:
            return True
        self.throw_error("The list [%s..] does not contain %s"
                   % (self._short(self.argument),
                      self._short(argument2)))

    def isTrue(self):
        self.validate(True)

    def isFalse(self):
        self.validate(False)

    def validate(self, expected=True):
        if self.argument is not None:
            if bool(self.argument) is expected:
                return True
            else: self.throw_error("%s is not %s" % (self._short(self.argument), expected))
        elif self.validation_function is not None:
            test_status, test_report = self.validation_function(*self.extra_arguments)
            if test_status is expected: return True
            self.throw_error(test_report)

    def throw_error(self, message):
        raise TestFailure("\r\n" * 2 + "-" * 40 + "\r\nTest Failed!!!\r\n"
                          + (message or "")
                          + "\r\n")

    def _short(self, arg, length=5):
        if hasattr(arg, "__iter__"):
            result = ", ".join([str(arg[i]) for i in xrange(0, min(len(arg), length))])
        else:
            result = str(arg)[:80]
        return result


def equals(argument1, argument2):
    if argument1 == argument2:
        return True, "%s  and  %s  are equal" % (argument1, argument2)
    return False, "%s  and  %s  are not equal" % (argument1, argument2)



#### just an example of usage
class MyTester(SmartTester):

    ## all those tests should fail

    def my_test_1(self):
        self.assertThat([1,4,5,6,7,9,0]).contains(2)

    def my_test_2(self):
        self.assertThat(2).equalsTo(3)

    def my_test_3(self):
        self.assertThat(equals, 2, 2).isFalse()

    def my_test_4(self):
        self.assertThat(False).isTrue()

    def my_test_5(self):
        self.assertThat(equals, 2, 3).isTrue()

